# -*- coding:utf-8 -*-
# @time:2023/2/109:28
# @author:LX
# @file:error.py
# @software:PyCharm

# -----


class SyntaxError(Exception):
    '''
        语法异常
    '''

class KeyError(Exception):
    '''
        字典的 Key 错误
    '''