# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'qssFile.py'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


