# -*- coding:utf-8 -*-
# @time:2022/12/128:53
# @author:LX
# @file:developmentLog.py
# @software:PyCharm
'''
    开发日志文件
'''


r'''
2022.12.13  0.0.1.4版本  气泡窗口已经基本实现,持续优化更新
2022.12.13  0.0.1.5版本  重写了气泡窗口的部分实现代码
2022.12.19  0.0.2.0版本
    --- 气泡窗口 优化了代码,修复了文字大小无法设置的BUG,以及文字位置的问题
    --- 新增无边框窗口 BorderlessWidget 类,导入方式 from PyQtGuiLib.core.widgets import BorderlessWidget
    --- 新增圆角窗口 RoundWidget 类,导入方式 from PyQtGuiLib.core.widgets import RoundWidget
    --- 靠边窗口功能完成(任意窗口都可以加上该功能),PullOver类 导入方式 from PyQtGuiLib.core.pullOver import PullOver
    --- 靠边窗口功能的测试用例已编写
2022.12.20  0.0.3.0版本
    --- 新增圆环进度条
2022.12.22  0.0.4.0版本
    --- 修复了一些小的BUG
    --- 放弃了亚克力窗口的研究
    --- 新增加载进度条
2022.12.22  0.0.5.0版本
    --- 新增水球进度条
    --- 新增进度条的集合测试
    --- 修改了气泡窗口的导入方式
         原来的导入方式: from PyQtGuiLib.core.bubbleWidget import BubbleWidget
         现在的导入方式: from PyQtGuiLib.core import BubbleWidget
    --- 修改了窗口靠边功能的导入方式
         原来的导入方式: from PyQtGuiLib.core.pullOver import PullOver
         现在的导入方式: from PyQtGuiLib.core import PullOver
2022.12.26  0.0.6.0版本
    --- 新增轮播组件,2022.12.22  已经基本完成,可能还存在BUG,以后修改(测试发布)
2022.12.26  0.0.6.1版本
    --- 新增轮播组件 新增两个方法,setAnimationTime(),setAinDirectionMode()
2022.12.26  0.0.7.0版本
    --- 新增线性渐变进度条,到目前为止,这是唯一个不提供文字显示的进度条
    --- 更新的所有进度条的抗锯齿写法 painter.setRenderHints(painter.Antialiasing | painter.SmoothPixmapTransform | painter.TextAntialiasing)
2023.1.4  0.0.8.0版本
    --- 线性渐变进度条 增加一个修改背景底色的方法 setBackGroundColor()
    --- 感谢[PyQt5学习爱好群-(讨厌自己)提供的BUG修复思路] 修复Borderless右下角拉伸BUG
    --- 新增 标题栏 TitleBar 类(测试发布中)
2023.1.4 0.0.8.1版本
    --- 修复 TitleBar 标题栏 在多个屏幕下的BUG
    --- TitleBar 在点击关闭按钮,新增隐退效果
    --- TitleBar 放大按钮动画增强
    --- TitleBar 增加缩小动画
2023.1.4 0.0.8.2版本
    --- TitleBar 更新Mac风格的放大,缩小,关闭按钮风格
    --- TitleBar 新增 setAniDuration() 方法 设置动画的时长
    --- TitleBar 新增 setTitleIcon(),setSyncWindowIcon() 与设置图标相关的方法
    --- 优化了TitleBar部分代码
2023.1.4 0.0.8.3版本
    --- 修复 TitleBar 类在mac下图标显示位置的问题
    --- utility.py 新增 系统判断
    --- 新增状态栏
    --- 状态栏 新增 setStatusPos() 方法
2023.1.5 0.0.8.3版本
    --- 优化 TitleBar,StatusBar 代码,并修复了样式表设置无效的BUG
2023.1.5 1.0.8.3版本
    --- 新增 styles 皮肤包项目 
    --- 全面兼容 PyQt5,PyQt6,PySide2,PySide6, PySide6 在使用无边框的还有小问题
2023.1.7 1.0.9.3版本
    --- 新增流式布局(FlowLayout)
    --- 互补色样式(contrastStyle())，同色调样式(homologyStyle())
    --- 修复 标题栏和状态栏 无法绘制文字的BUG
2023.1.8 1.0.9.4版本
    --- 修复流式布局 在mac的间距问题
    --- 优化 标题栏 和 状态栏 的代码
    --- 优化获取文字大小函数 textSize() 对丢失像素进行补偿
    --- 优化标题栏 在PyQt6,PySide6,标题高度计算不精准的Bug
2023.1.8 1.1.9.4版本
    --- 感谢! PyQt5学习爱好群-(讨厌自己)修复 PyQt6 版本下 无边框类(Borderless)移动BUG
    --- 增加 内置-样式设计器,主要是为了配合皮肤包
2023.1.9 1.1.9.5版本
    --- 标题栏和状态栏 终于 不用主窗口中重写 resizeEvent事件来调用 updateXXX() 来完成大小自适应了
    --- 修复了 内置-样式设计器 不少BUG
2023.1.9 1.2.10.5版本
    --- 新增 控件的组成分析函数 导入方式 from PyQtGuiLib.core.resolver import dumpStructure
    --- 新增 QListWidget 增强版本 - ListWidget
    --- 轮播组件重写中
2023.1.12 - 2023-1.13 [1.2.12.5] 版本
    --- 删除 文件buttonWidget.py 和 roundWidget.py
    --- 重构的 无边框的代码 增加了许多功能,请看文档
    --- 标题栏 新增了 窗口钉住功能, Win 和 Mac 各有一套风格
    --- 新增功能 滚动栏(RollWidget)
    --- ListWidget 新增一种动画效果
    --- 新增 说明 流式布局 无法 配合 QScrollArea 使用
2023.1.14 1.2.13.6
    --- 重新修改了标题栏的继承方式
    --- 更新 无边框窗口 的代码,修复无边框窗口在作为子窗口时,不显示的BUG,
        并失去做为主窗口时的部分功能
    --- 新增了一个 动态标题输入框(DynamicTLine)
    --- 对流式布局进行了优化
    --- 移除了 内置样式设计器的 controlConfig.json 改为 controlConfig.py
2023.1.15 1.2.15.6
    --- 新增开关按钮(SwitchButton)
    --- 新增调色版(ColorPalette)
2023.1.15 1.2.15.7
    --- 调色版相关问题
            1.修复中间区域无法获取颜色的BUG
            2.修复中间区域点击无法触发信号BUG
            3.新增一个获取当前选中颜色的按钮
            4.新增两个信号(colorNamed,clicked)
            5.新增两种风格(Style_Black,Style_White)
    --- 重构 气泡窗口
2023.1.20 [1.2.17.7]
    --- 气泡窗口 重构完成,并使用使用了全新的QSS来设置样式
    --- 轮播组件 重构完成
    --- 以及更新部分测试用例
2023.1.21 [1.2.17.8]
    --- 状态栏 无法实时跟随窗口,需要手动的调用 updateStatusSize()
# ====== 从 2.2.17.8 版本开始 所有的样式将有全新的自定义QSS来代替
2023.1.31 [2.2.17.8]
    --- 重构了整个widgets项目,将原本的widgets移动到abandonCase目
        -- 目前只有 borderlessWidget 可用,主窗口,标题栏,状态栏,还在编写中
    --- 从 2.2.17.8这个版本开始所有的样式将有全新的自定义QSS来代替,并且不兼容前面的版本
        --- 支持自定义QSS的已经在文档中标出
    --- 优化了部分控件的代码,删除了一些不常用的控件,和测试用例
    --- 修改了大部分控件的继承对象
2023.2.9 [2.3.17.8]
    --- 感谢! PyQt5学习爱好群-(迷糊蛋der～)在2.2.17.8版本在中提出的qt版本,代码上的兼容性问题
    --- 修复的pyside兼容性的问题,
    --- 新增 QSS解析器 QssStyleAnalysis 
2023.2.13 - 2023.2.15 [2.3.18.9]
    --- 修复 QSS解析器 在追加新的属性时会覆盖原来相同属性的BUG. 目前
        追加新的属性时,会与原来的相同属性进行融合
    --- 新增 功能函数 loadUic 统一四个版本加载 UI文件 的函数,对外提统一的接口
    --- 新增滑块(Slider)

2023.2.18 - 2023.2.20 [2.4.18.11]
    --- 修改原来导入导入颜色板的方式
        原来: from PyQtGuiLib.core import ColorPalette (与这个相关文件已删除)
        现在: from PyQtGuiLib.core import PaletteFrame
        并更新代码,将原来的代码拆分为,获取颜色的框架,与独立的颜色区域,
        在框架中可以嵌入不同的颜色区别,目前支持(Rect,Wheel)
    --- 修复 QSS解析器 已知BUG,
        修复 isSelectKey() 取值成功,但是值为0,导致整个函数返回False的BUG
        新增 header(),parent()方法
        并新增对 无选择器的QSS语法  
                  color: rgb(0, 255, 127);
                  background-color:rgb(0, 170, 0);
        进行处理,该语法只能用过setQSS方法设置
        新增 QSS解析器例子,位置 PyQtGuiLib -> tests -> test_QssStyleAnalysis -> eg6.py
    --- 动态样式链接器 from PyQtGuiLib.styles import StyleLinker
        目前仅支持 QPushButton,QLabel,QLineEdit 控件
        
2023.2.20 - 2023.3.23 [2.5.18.11]
    调整结构 将 abandonCasem目录,Log目录和tests目录移出项目
    新增ToolListWidget 控件(未完成)
    新增SuperPainter 超级画师(未完成) - 重构中
    新增Animation 动画框架(未完成) - 测试部分

2023.3.24 - 2023.4.7
    - Animation 动画框架 已经快接近完成,目前大部分功能已经可用
    - 修改QSS解析器的已知BUG,并且在多线程时,是安全的
    - 新增 SuperPainter 超级画师类
        - 导入方法 from PyQtGuiLib.styles import SuperPainter
        - 教学案例在test\test_SuperPainter 目录下
2023.4.7 - 2023.4.11
    - 更新模板窗口 ListTemplateWindow
2023.4.13 
     感谢! PyQt5学习爱好群-(讨厌自己) 贡献 CircularProgressBar 进度条模块
     导入方式 from PyQtGuiLib.core.progressBar import CircularProgressBar
2023.4.20
    - 新增 抽屉控件(Drawer) -- 编写中
2023.4.21 
    -抽屉控件(Drawer)完成
2023.4.25
    -修复PySide2导入pyqtProperty的BUG
    -重构了动画框架,并删除老代码以及老的测试用例,新的动画案例还未编写
    -删除styles包下整个linker项目,以及测试用例
    -所有由其他人贡献的功能模块里面都会有一个author()方法来记录作者信息
    -新增控件ComboCheckBox,该控件是一个具体提示的输入框,同时也是一个多项选择框,
        控件由 "PyQt5学习爱好群-知行合一" 贡献
        该控件目前PySide系列支持性比较好,PyQt系列存在提示BUG
        导入方式 from PyQtGuiLib.core import ComboCheckBox
    -删除老版本的超级画师代码,重构版本的超级画师目前代码提示还不完善
    -目前所有文档还未更新,
2023.4.26
    新增控件 PageCuttingButtonGroup 分页组件
        控件由 "PyQt5学习爱好群-(讨厌自己)" 贡献
        导入方式 from PyQtGuiLib.core import PageCuttingButtonGroup
2023.4.28
    - 修改协议 MIT -> Apache License 2.0
2023.5.2
    - 修复Drawer抽屉控件,添加QListWidget的BUG
'''


# -------------------------------
'''
    暂时搁浅的任务
1.新无边框窗口主窗口,(还没有设计完成,暂时先放下 2023.1.31)
'''

# ===============================================

'''
记录已知BUG,但是还未修复
2023.2.18 
    LoadBar(加载进度条) -- 有BUG,窗口变化时,进度条不会跟随变化

2023.3.23 
        动画框架---
        出现BUG,当动画同时操作一个控件,不同样式状态下相同属性时出现BUG
'''