# -*- coding:utf-8 -*-
# @time:2022/12/1211:40
# @author:LX
# @file:__init__.py.py
# @software:PyCharm
from PyQtGuiLib.animation.animation import (
    Animation,
    ParallelAnimationGroup,
    SequentialAnimationGroup
)
